/**************************************************************************** 
*  Copyright 2008 C&H Technologies, Inc.  All Rights Reserved.					    * 
*****************************************************************************/
/****************************************************************************
*
* m220_ansi.h  
*
*	Header file for the M220 ANSI-C source module.
*
*	C&H Part Number: 11029557
*
*****************************************************************************/
#ifndef	M220_HEADER
#define	M220_HEADER

#ifndef   PLATFORM
#define		PLATFORM chwin
#endif

#include "apis.h"

/***************************
  	Type Definitions
****************************/
typedef char Int8;
typedef unsigned char UInt8;
typedef short Int16;
typedef unsigned short UInt16;
typedef int Int32;
typedef unsigned int UInt32;
typedef unsigned short Bool;
typedef double Real64;

typedef struct 
{ APIS_HANDLE apis_handle;
	UInt32			apis_error;
	Bool overlap;
	Bool fifo_full_wait;
	UInt16 timeout_ms;
}M220_INSTR, *M220_HANDLE;

/**************************************************************************** 
*  Error Codes
*****************************************************************************/
#define		M220_SUCCESS														0
#define		M220_SPECIFIC_ERROR_BASE								0xBFFC0900
#define		M220_ERROR_WAIT_TIMEOUT_EXPIRED					(M220_SPECIFIC_ERROR_BASE + 0)
#define		M220_ERROR_PARAMETER1										(M220_SPECIFIC_ERROR_BASE + 1)
#define		M220_ERROR_PARAMETER2										(M220_SPECIFIC_ERROR_BASE + 2)
#define		M220_ERROR_PARAMETER3										(M220_SPECIFIC_ERROR_BASE + 3)
#define		M220_ERROR_PARAMETER4										(M220_SPECIFIC_ERROR_BASE + 4)
#define		M220_ERROR_FAIL_ID_QUERY								(M220_SPECIFIC_ERROR_BASE + 5)
#define		M220_ERROR_APIS													(M220_SPECIFIC_ERROR_BASE + 6)
#define		M220_ERROR_OUT_OF_MEMORY								(M220_SPECIFIC_ERROR_BASE + 7)
#define		M220_ERROR_INVALID_ATTRIBUTE						(M220_SPECIFIC_ERROR_BASE + 8)
#define		M220_SPECIFIC_ERROR_END									(M220_SPECIFIC_ERROR_BASE + 9)
#define		M220_SPECIFIC_ERROR_ABSOLUTE_MAX				(M220_SPECIFIC_ERROR_BASE + 0x1FFF)

/**************************************************************************** 
*  Warning Codes
*****************************************************************************/
#define		M220_SPECIFIC_WARNING_BASE							0x3FFC0900
#define		M220_SPECIFIC_WARNING_END								(M220_SPECIFIC_WARNING_BASE + 1)
#define		M220_SPECIFIC_WARNING_ABSOLUTE_MAX			(M220_SPECIFIC_WARNING_BASE + 0x6FF)

#define		M220_TRUE								1
#define		M220_FALSE							0

#define		M220_ATTR_OVERLAP_MODE				0
#define		M220_ATTR_TIMEOUT							1
#define		M220_ATTR_RESERVED						2
#define		M220_ATTR_FIFO_FULL_WAIT			3

/**************************************************************************** 
*	M220 Register offsets
*****************************************************************************/
#define		M220_STATUS_REGISTER										0x00
#define		M220_CONTROL_REGISTER										0x02
#define		M220_ROW0_SET_REGISTER									0x10
#define		M220_ROW0_RESET_REGISTER								0x12
#define		M220_ROW1_SET_REGISTER									0x14
#define		M220_ROW1_RESET_REGISTER								0x16
#define		M220_ROW2_SET_REGISTER									0x18
#define		M220_ROW2_RESET_REGISTER								0x1A
#define		M220_ROW3_SET_REGISTER									0x1C
#define		M220_ROW3_RESET_REGISTER								0x1E
#define		M220_ID_ACCESS_REGISTER								  0xFE


/**************************************************************************** 
*	Bit Definitions
*****************************************************************************/
/*  Status Register  */
#define		M220_INTERRUPT_STATUS_BIT										0x0001	 // 1, Indicates Interrupt is Asserted
#define		M220_FIFO_FULL_STATUS_BIT										0x0002	 // 1, Indicates is FIFO currently full
#define		M220_FIFO_EMPTY_STATUS_BIT									0x0004	 // 1, Indicates FIFO is currenlty empty
#define		M220_MPS_BIT																0x0008   // 1 -> dual 8x1 mode, 0 -> single 16x1 mode
#define		M220_INIT_STATUS_BIT												0x0010	 // 1, Indicates M220 is Initialized

/*	Control Register */
#define   M220_CONTROL_RESET_BIT                      0x0001   // 1 -> initiates a soft reset, 0 -> releases module from reset condition)
#define   M220_CONTROL_INTERRUPT_ENABLE_BIT						0x0002	 // 1 -> enables interrupt line
#define   M220_CONTROL_SELFTEST_ENABLE_BIT            0x0004	 // 1 -> self test
#define   M220_CONTROL_DRIVER_POWER_ENABLE_BIT				0x0008	 // 1 -> enable relay power, 0 -> removes power
#define   M220_CONTROL_TIMER_BITS											0x0030	 // 00 -> 8ms, 01 -> 2ms, 10 -> 4ms, 11 -> 64ms

/**************************************************************************** 
*	Parameter Ranges
*****************************************************************************/
#define		M220_MIN_TIME_MODE						0x00
#define   M220_MAX_TIME_MODE						0x11

/**************************************************************************** 
*  M-Module ID Words
*****************************************************************************/
#define		M220_IDENT_WORD_SYNC_CODE			     0
#define		M220_IDENT_WORD_MODULE_NUM				 1
#define		M220_IDENT_WORD_REVISION_NUM 		   2
#define		M220_IDENT_WORD_MODULE_CHAR 			 3
#define		M220_IDENT_WORD_VXI_SYNC_CODE 		16
#define		M220_IDENT_WORD_VXI_ID 					  17
#define		M220_IDENT_WORD_VXI_DEVICE_TYPE   18


#define		M220_VXI_MANF_ID									0xFFF
#define		M220_VXI_MODEL_CODE							  0x25D


/**************************************************************************** 
*	Exported Function Prototypes
*****************************************************************************/
Int32 m220_SetTimerMode(M220_HANDLE handle, UInt16 mode);
Int32 m220_EnableRelayPower(M220_HANDLE handle, Bool enable);
Int32 m220_EnableSelfTest(M220_HANDLE handle, Bool enable);
Int32 m220_EnableInterrupts (M220_HANDLE handle, Bool enable);
Int32 m220_Reset(M220_HANDLE handle);

Int32 m220_InitStatus(M220_HANDLE handle, Bool *Status);
Int32 m220_FIFOEmptyStatus(M220_HANDLE handle, Bool *Status);
Int32 m220_FIFOFullStatus(M220_HANDLE handle, Bool *Status);
Int32 m220_InterruptAssertedStatus(M220_HANDLE handle, Bool *status);
Int32 m220_ChannelStatus(M220_HANDLE handle, UInt16 channel, Bool *Status);
Int32 m220_OpcQuery(M220_HANDLE handle, Int16 *state);
Int32 m220_GetMUXMode(M220_HANDLE handle, Bool *mode);

Int32 m220_GetAttribute(M220_HANDLE handle, Int16 attributeName, Int32 *attributeValue);
Int32 m220_SetAttribute( M220_HANDLE handle, Int16 attributeName, Int32 attributeValue);
Int32 m220_Overlap(M220_HANDLE handle, Bool overlap);
Int32 m220_OverlapQuery(M220_HANDLE handle, Bool *overlap);
Int32 m220_RouteCloseQuery(M220_HANDLE handle, Int16 channel, Bool *closed);
Int32 m220_RouteClose(M220_HANDLE handle, UInt16 channel);
Int32 m220_RouteCloseList(M220_HANDLE handle, char *list);
Int32 m220_RouteCloseListQuery(M220_HANDLE handle, char *channel_list, char *route_close_list);
Int32 m220_RouteOpenQuery(M220_HANDLE handle, Int16 channel, Bool *opened);
Int32 m220_RouteOpen(M220_HANDLE handle, UInt16 channel);
Int32 m220_RouteOpenList(M220_HANDLE handle, char *list);
Int32 m220_RouteOpenListQuery(M220_HANDLE handle, char *channel_list, char *route_open_list);
Int32 m220_SetAllQuery(M220_HANDLE handle, Int16 *state);
Int32 m220_SetAll(M220_HANDLE handle, Int16 state);

/*  Utility Functions  */
Int32 m220_RevisionQuery(INT8 driverRev[]);
Int32 m220_ErrorMessage(Int32 statusCode, INT8 errorMessage[]);
Int32 m220_GetIdentWord(M220_HANDLE handle, UInt16 offset, UInt16 *value);

/*****************************************************************************
*	Modify the functions below this point to use the M220 driver in an 
*	environment not supported by an APIS platform support module. All functions
*	above this point should not have to be modified.
*
*	In general, any call to an APIS function (apis_func()) will need to be
*	modified to communicate with the instrument or operating system being used.
*
******************************************************************************/
Int32 m220_Init(Int32 instr, Bool IDQuery, Bool reset, M220_HANDLE *handle);
Int32 m220_Close(M220_HANDLE handle);
Int32 m220_ReadRegister(M220_HANDLE handle, UInt16 offset, UInt16 *value);
Int32 m220_WriteRegister(M220_HANDLE handle, UInt16 offset, UInt16 value);
Int32 m220_InstallIrq(M220_HANDLE handle, void *interruptHandler, Int32 vector, Int32 level, Int32 mode, void *var_ptr);
Int32 m220_UninstallIrq(M220_HANDLE handle);
Int32 m220_GetApisError(M220_HANDLE handle, UInt32 *apis_error);

#endif		//M220_HEADER
